package WebService::Strava::Athlete::Gear::Bike;

use v5.010;
use strict;
use warnings;
use Moo;

extends 'WebService::Strava::Athlete::Gear';

# ABSTRACT: An Athlete's Bike

our $VERSION = '0.05'; # VERSION: Generated by DZP::OurPkg:Version


has 'frame_type' => ( is => 'ro' );

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

WebService::Strava::Athlete::Gear::Bike - An Athlete's Bike

=head1 VERSION

version 0.05

=head1 SYNOPSIS

  Provides a Bike object

=head1 DESCRIPTION

  Though currently gear items can only be retrieved if Strava
  extend the API to be able to update/change/remove gear, this
  will provide the framework to easily extend the library.

=head1 AUTHOR

Leon Wright < techman@cpan.org >

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2014 by Leon Wright.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
