
BEGIN {
  unless ($ENV{AUTHOR_TESTING}) {
    require Test::More;
    Test::More::plan(skip_all => 'these tests are for testing by the author');
  }
}

use strict;
use warnings;
use Test::More;

# generated by Dist::Zilla::Plugin::Test::PodSpelling 2.006008
use Test::Spelling 0.12;
use Pod::Wordlist;


add_stopwords(<DATA>);
all_pod_files_spelling_ok( qw( bin lib  ) );
__DATA__
strava
Strava
oauth
api
Fenwick
Maint
Moyer
leaderboard
cli
GPX
alpineski
backcountryski
gpx
iceskate
ie
inlineskate
instantiation
kitesurf
nordicski
rollerski
tcx
Leon
Wright
techman
lib
WebService
Club
Athlete
Segment_Effort
Activity
Gear
Bike
Shoe
Auth
Segment
